/*
 *  jDTAUS Core RI Client Container
 *  Copyright (c) 2005 Christian Schulte
 *
 *  Christian Schulte, Haldener Strasse 72, 58095 Hagen, Germany
 *  <cs@jdtaus.org> (+49 2331 3543887)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.container.ri.client;

import java.lang.ref.Reference;
import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.Map;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.Messages;
import org.jdtaus.core.container.Properties;

/**
 * Instance meta-data.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: Instance.java 5416 2008-10-27 10:55:51Z schulte2005 $
 */
class Instance
{

    /** Scope the instance applies to. */
    private int scope;

    /** Class name of the instance. */
    private String className;

    /** Model version of the instance. */
    private String modelVersion;

    /** Map of dependency objects of the instance. */
    private Map dependencies;

    /** Implementation meta-data of the instance. */
    private Implementation implementation;

    /** Dependency meta-data of the instance. */
    private Dependency dependency;

    public Instance( final int scope, final String className,
                     final String modelVersion )
    {
        this.scope = scope;
        this.className = className;
        this.modelVersion = modelVersion;
    }

    public int getScope()
    {
        return this.scope;
    }

    public String getClassName()
    {
        return this.className;
    }

    public String getModelVersion()
    {
        return this.modelVersion;
    }

    public Implementation getImplementation()
    {
        return this.implementation;
    }

    public void setImplementation( final Implementation implementation )
    {
        this.implementation = implementation;
    }

    public Dependency getDependency()
    {
        return this.dependency;
    }

    public void setDependency( final Dependency dependency )
    {
        this.dependency = dependency;
    }

    public Properties getProperties()
    {
        Properties properties = null;

        if ( this.implementation != null )
        {
            properties = this.implementation.getProperties();
        }
        else if ( this.dependency != null )
        {
            properties = this.dependency.getProperties();
        }

        return properties;
    }

    public Messages getMessages()
    {
        Messages messages = null;

        if ( this.implementation != null )
        {
            messages = this.implementation.getMessages();
        }
        else if ( this.dependency != null )
        {
            messages = this.dependency.getImplementation().getMessages();
        }

        return messages;
    }

    public Object getDependencyObject(
        final String dependencyName )
    {
        if ( this.dependencies == null )
        {
            this.dependencies = new HashMap();
        }

        Object instance = null;
        final Reference ref =
            (Reference) this.dependencies.get( dependencyName );

        if ( ref != null )
        {
            instance = ref.get();
            if ( instance == null )
            {
                this.dependencies.remove( dependencyName );
            }
        }

        return instance;
    }

    public void setDependencyObject( final String dependencyName,
                                                  final Object object )
    {
        if ( this.dependencies == null )
        {
            this.dependencies = new HashMap();
        }

        this.dependencies.put( dependencyName, new WeakReference( object ) );
    }
}
